-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;

local masterLooter = "";


-----------------------
----LOCAL FUNCTIONS----
-----------------------


--Function for transmitting the tradeable item to the leader/player's itemList.
local function sendTradeableFunc(self)
	local input = self.editBox:GetText();

	if UnitPlayerOrPetInRaid(input) then
		local _, link = GetItemInfo(self.data);
		
		if input == variables.playerName then
			functions.registerItem(input, link);
		else
			SendChatMessage(link, "WHISPER", nil, input);
		end

		masterLooter = input;
	else
		functions.printOut("Player is not in the raid group.");
	end
end

--Function for handling manual rolls.
local function manualRoll(self)
	local number = tonumber(self.editBox:GetText());

	if number then
		functions.hideUser();
		functions.disableLeader();
		
		RandomRoll(1,number);
	end

	self:Hide();
end

--Function for checking if the player is on the same realm. If not then change the name of roller to be their name + realm.
local function realmCheck(roller)
	if UnitRealmRelationship(roller) ~= 1 then
		local name, realm = UnitName(roller);
		roller = name .. "-" .. realm;
	end

	return roller;
end

--Function for sending the whisper notification.
local function sendWhisperNotification(owner, winner, item)
	--Make sure owner exists.
	if not owner then
		functions.printOut('Error, no owner has been recorded. Make sure raid warning has been posted in the correct syntax (e.g. "NAME - ITEMLINK")');
		return;
	end

	--If the winner is the original owner of the item, then tell them to keep it.
	if winner == owner then
		SendChatMessage("You may keep " .. item .. ".", "WHISPER", "Common", owner);
		return;
	end
	
	--Edit the names to include the realm name if they are on a different server than you.
	owner = realmCheck(owner);
	winner = realmCheck(winner);

	--Make sure both players are in the raid group.
	if not (UnitPlayerOrPetInRaid(owner) and UnitPlayerOrPetInRaid(winner)) then
		functions.printOut("Owner or winner is not in the raid group.");
		return;
	end

	--Check if the distributer is the winner of the item, if so then don't send a trade message to yourself.
	if winner ~= variables.playerName then
		SendChatMessage("Trade " .. owner .. " for " .. item .. "." , "WHISPER", "Common", winner);
	end

	--Check if the distributer is the owner of the item, if so then don't send a give message to yourself.
	if owner ~= variables.playerName then
		SendChatMessage("Give " .. item .. " to " .. winner .. ".", "WHISPER", "Common", owner);
	end
end

--Function for sending winenr in raid warning.
local function sendRaidWarningNotification(owner, winner, item, rollType)
	local itemName = GetItemInfo(item);

	if owner == winner then
		SendChatMessage(owner .. " keeps " .. itemName .. ".", "RAID_WARNING", nil, nil);
	else
		SendChatMessage(winner .. " wins " .. itemName .. (owner and (" from " .. owner) or "") .. " for " .. rollType .. " roll.", "RAID_WARNING", nil, nil);
	end
end


-----------------------
-----POPUP WINDOWS-----
-----------------------


--Pop up for manual rolls.
StaticPopupDialogs["LPTLootRoll_ManualRoll"] = 
{
	text = "Enter roll (Max: 9999):",
	button1 = "Accept",
	button2 = "Cancel",
	hasEditBox = 1,
	maxLetters = 4,
	OnShow = 
	function(self)
		local editBox = self.editBox;
		editBox:SetText("");
		editBox:SetFocus();
	end,
	OnAccept = 
	function(self)
		manualRoll(self);
	end,
	EditBoxOnEnterPressed = 
	function(self)
		manualRoll(self:GetParent());
	end,
	EditBoxOnEscapePressed = 
	function(self)
		self:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting history.
StaticPopupDialogs["LPTLootRoll_ClearHistory"] = 
{
	text = "Are you sure you want to reset roll history?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self)
		functions.resetItemAndRollHistory();
		functions.printOut("LPT Loot Roll: Item history has been reset.");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting locations.
StaticPopupDialogs["LPTLootRoll_ResetPosition"] = 
{
	text = "Are you sure you want to reset window positions?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self)
		functions.resetUserWindow();
        functions.resetLeaderWindow();
        functions.resetItemListWindow();
		functions.printOut("LPT Loot Roll: Location of windows have been reset.");
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for sending unwanted loot.
StaticPopupDialogs["LPTLootRoll_SendTradeable"] =
{
	text = "Do you want to list %s for trade? If accepted itemlink will be sent to the player below.",
	button1 = "Yes",
	button2 = "No",
	hasItemFrame = 1,
	hasEditBox = 1,
	maxLetters = 24,
	OnShow = 
	function(self)
		local editBox = self.editBox;

		if llrSettings.masterLooterMode then
			masterLooter = llrSettings.masterLooter;
		end

		if masterLooter == "" or not (UnitPlayerOrPetInRaid(masterLooter) and (UnitIsGroupAssistant(masterLooter) or UnitIsGroupLeader(masterLooter))) then
			masterLooter = functions.getRaidLeader() or "";
		end

		editBox:SetPoint("BOTTOM", 0, 90);
		editBox:SetText(masterLooter);
		editBox:SetAutoFocus(false);
		editBox:ClearFocus();
	end,
	OnAccept = 
	function(self)
		sendTradeableFunc(self);
	end,
	EditBoxOnEnterPressed = 
	function(self)
		sendTradeableFunc(self:GetParent());
	end,
	EditBoxOnEscapePressed = 
	function(self)
		self:ClearFocus();
	end,
	OnHide = function (self, data)
		functions.popTradeable();
	end,
	timeout = 0,
	whileDead = 1,
	noCancelOnReuse = 1,
	hideOnEscape = 1
};

--Pop up for accepting settings rolls.
StaticPopupDialogs["LPTLootRoll_SettingShare"] = {
	text = "Are you sure you want to change roll values?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self, msg)
		--If user accepts the data, parse it to numbers and store it in settings.
		local main, off, mog = strsplit("-", msg);

		main = tonumber(main);
		off  = tonumber(off);
		mog  = tonumber(mog);
		
		if main and off and mog then
			llrSettings.mainRoll = main;
			llrSettings.offRoll = off;
			llrSettings.mogRoll = mog;
		end
		
		functions.printOut("Roll values have been changed to:")
		functions.printOut("Main: " .. main);
		functions.printOut("Off: " .. off);
		functions.printOut("Mog: " .. mog);
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for distributing loot.
StaticPopupDialogs["LPTLootRoll_DistributeItem"] =
{
	text = "Give item to %s?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = 
	function(self, msg)
		local winner, rollType = strsplit("-", msg);

		if not winner or not rollType then
			return;
		end

		local owner, item = variables.rollHistory[1].owner, variables.rollHistory[1].item;

		variables.rollHistory[1].distributed = true;

		--Close leader window unless whisperlistener is enabled and there are more items to distribute.
		if not (llrSettings.whisperListener and #variables.itemsToRoll > 0) then
			functions.closeLeaderWindow();
		end

		if llrSettings.whisperNotificationMode then
			sendWhisperNotification(owner, winner, item);
		else
			sendRaidWarningNotification(owner, winner, item, rollType);
		end
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};